
--   $ot_monitor_clientpb = q{

Create or replace package body ot_monitor_client is

/* Copyright 2002, Adam vonNieda */

   /* Globals */

   vDebug		NUMBER;
   vInstanceName	VARCHAR2(16);

   Procedure get_instance

/* Get the instance name. */

   (
      vInstanceName	OUT VARCHAR2
   )

   IS

      CURSOR instance_name IS
      SELECT INSTANCE_NAME FROM V$INSTANCE;

   BEGIN

      OPEN instance_name;
      FETCH instance_name INTO vInstanceName;
      CLOSE instance_name;

   END;

   Procedure send_alert

/* 
   Check that the comminication between client and server is OK.
      Exception if not.
   Send a message to the server containing alert information. 
*/

   (

      pResourceType	IN VARCHAR2,
      pResourceName	IN VARCHAR2,
      pAlert		IN VARCHAR2

   )

   IS

      vConnection	NUMBER;

   BEGIN

       get_instance(vInstanceName);
       check_connection(vDebug,vConnection);

       debug_message('Sending alert: '||vInstanceName||' : Type: '||pResourceType||' Name: '||pResourceName||' Alert: '||pAlert);

       INSERT INTO OT_MONITOR_ALERTS@OT_MONITOR (CLIENT,RESOURCE_TYPE,RESOURCE_NAME,ALERT)
          VALUES (vInstanceName,pResourceType,pResourceName,pAlert);

   END;
       
   Procedure alertlog_message

/* Write a message to the alert log. */

   (
      pText		IN VARCHAR2
   )
   is

   BEGIN
      IF vDebug > 0 THEN
         sys.dbms_system.ksdwrt(2,'Oracletool: '||pText);
      END IF;
   END;

   Procedure debug_message

/* Write a message to stdout */

   (
      pText		IN VARCHAR2
   )
   is

   BEGIN
      IF vDebug > 0 THEN
         dbms_output.enable(1000000);
         dbms_output.put_line('Debug: ('||vInstanceName||') '||pText);
         sys.dbms_system.ksdwrt(2,'Oracletool: '||pText);
      END IF;
   END;

   Procedure check_connection

/*

      Purpose	: Check for a valid database link to the server.

*/

      (
         pDebug			IN NUMBER DEFAULT 0,
         vReturnCode		OUT NUMBER
      )

      IS

---------------------------------------------------------------------------------------------------------------------
--  Declarations
---------------------------------------------------------------------------------------------------------------------

      vLinkCount		NUMBER;

      CURSOR dblink IS
      SELECT
         COUNT(*) FROM USER_DB_LINKS
      WHERE DB_LINK LIKE 'OT_MONITOR%';

      CURSOR connectivity IS
      SELECT
          COUNT(*) FROM DUAL@OT_MONITOR;

      no_server_connection EXCEPTION;
      PRAGMA EXCEPTION_INIT(no_server_connection,-20000);

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set debug level which may have been passed.
---------------------------------------------------------------------------------------------------------------------

         vDebug := pDebug;

         debug_message('Checking for database link OT_MONITOR');

         OPEN dblink;
         FETCH dblink INTO vReturnCode;
         CLOSE dblink;
         IF vReturnCode > 0 THEN
            debug_message('Database link was found');
         ELSE
            debug_message('Database link was NOT found');
            RAISE no_server_connection;
         END IF;

         debug_message('Checking for connectivity over database link OT_MONITOR');

         OPEN connectivity;
         FETCH connectivity INTO vReturnCode;
         CLOSE connectivity;
         IF vReturnCode > 0 THEN
            debug_message('Database link works');
         ELSE
            debug_message('Database link does NOT work');
            RAISE no_server_connection;
         END IF;

      END;

   Procedure extent_max

/*

      Purpose	: Check for objects which have an extents >  (MAX_EXTENTS - threshold)

*/

      (
         pThreshold		IN NUMBER,
         pDebug			IN NUMBER DEFAULT 0
      )
      is

---------------------------------------------------------------------------------------------------------------------
--  Declarations
---------------------------------------------------------------------------------------------------------------------

      vSegmentName		DBA_SEGMENTS.SEGMENT_NAME%TYPE;
      vSegmentType		DBA_SEGMENTS.SEGMENT_TYPE%TYPE;
      vOwner			DBA_SEGMENTS.OWNER%TYPE;
      vExtentCount		VARCHAR2(50);
      vExtentMax		VARCHAR2(50);
      vTablespaceName		DBA_SEGMENTS.TABLESPACE_NAME%TYPE;
      vThreshold		NUMBER;
      vMaxMinusThres		NUMBER;

      CURSOR extentCursor IS
      SELECT
         SEGMENT_NAME,
         SEGMENT_TYPE,
         OWNER,
         LTRIM(TO_CHAR(EXTENTS,'999,999,999,999')),
         LTRIM(TO_CHAR(MAX_EXTENTS,'999,999,999,999')),
         TABLESPACE_NAME
      FROM DBA_SEGMENTS
         WHERE EXTENTS > (MAX_EXTENTS-vThreshold)
         AND SEGMENT_TYPE != 'CACHE';

      CURSOR threshold IS
      SELECT 
         THRESHOLD FROM OT_MONITOR_THRESHOLDS@OT_MONITOR
      WHERE CLIENT = vInstanceName
      AND RESOURCE_TYPE = 'EXTENTMAX';

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set debug level which may have been passed. Otherwise, use the global.
---------------------------------------------------------------------------------------------------------------------

      IF pDebug > 0 THEN
         vDebug := pDebug;
      END IF;

---------------------------------------------------------------------------------------------------------------------
-- Get the instance name.
---------------------------------------------------------------------------------------------------------------------

      get_instance(vInstanceName);

---------------------------------------------------------------------------------------------------------------------
-- Find out if a threshold value is passed, else we will use default.
---------------------------------------------------------------------------------------------------------------------

      IF pThreshold IS NOT NULL THEN
         vThreshold := pThreshold;
      ELSE
         OPEN threshold;
         FETCH threshold INTO vThreshold;
         CLOSE threshold;
      END IF;

      debug_message('Checking for objects which have extents > '||vThreshold);

      open extentCursor;
      LOOP
         fetch extentCursor into vSegmentName,vSegmentType,vOwner,vExtentCount,vExtentMax,vTablespaceName;
         exit when extentCursor%NOTFOUND;
         IF vExtentCount >= vExtentMax-vThreshold THEN
            vMaxMinusThres := vExtentMax-vThreshold;
            alertlog_message(vSegmentType||' '||vOwner||'.'||vSegmentName||' has '||vExtentCount||' extents. Approaching max of '||vExtentMax||'.  Tablespace '||vTablespaceName||'. Threshold is '||vThreshold);
            send_alert('SEGMENTMAX',vOwner||'.'||vSegmentName,vSegmentType||' '||vOwner||'.'||vSegmentName||' has '||vExtentCount||' extents. Approaching max of '||vExtentMax||' in tablespace '||vTablespaceName||'. Threshold is max_extents - '||vThreshold||' ('||vMaxMinusThres||').');
         END IF;
      END LOOP;
      close extentCursor;
   END;

   Procedure extent_count

/*

      Purpose	: Check for objects which have an extent count > threshold

*/

      (
         pThreshold		IN NUMBER,
         pDebug			IN NUMBER DEFAULT 0
      )
      is

---------------------------------------------------------------------------------------------------------------------
--  Declarations
---------------------------------------------------------------------------------------------------------------------

      vSegmentName		DBA_EXTENTS.SEGMENT_NAME%TYPE;
      vSegmentType		DBA_EXTENTS.SEGMENT_TYPE%TYPE;
      vOwner			DBA_EXTENTS.OWNER%TYPE;
      vSegmentCount		NUMBER;
      vSegmentSize		VARCHAR2(50);
      vTablespaceName		DBA_EXTENTS.TABLESPACE_NAME%TYPE;
      vThreshold		NUMBER;
      vExceeded			NUMBER;

      CURSOR extentCursor IS
      SELECT
         SEGMENT_NAME,
         SEGMENT_TYPE,
         OWNER,
         TO_CHAR(COUNT(*),'999,999,999,999'),
         LTRIM(TO_CHAR(SUM(BYTES),'999,999,999,999')),
         TABLESPACE_NAME
      FROM DBA_EXTENTS
         WHERE SEGMENT_TYPE IN ('CLUSTER','INDEX','INDEX PARTITION','LOBINDEX','LOBSEGMENT','TABLE','TABLE PARTITION','ROLLBACK','UNDO')
         HAVING COUNT(*) >= vThreshold
         GROUP BY SEGMENT_TYPE, SEGMENT_NAME, TABLESPACE_NAME, OWNER
         ORDER BY 4 DESC;

      CURSOR threshold IS
      SELECT 
         THRESHOLD FROM OT_MONITOR_THRESHOLDS@OT_MONITOR
      WHERE CLIENT = vInstanceName
      AND RESOURCE_TYPE = 'EXTENTCOUNT';

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set debug level which may have been passed. Otherwise, use the global.
---------------------------------------------------------------------------------------------------------------------

      IF pDebug > 0 THEN
         vDebug := pDebug;
      END IF;

---------------------------------------------------------------------------------------------------------------------
-- Get the instance name.
---------------------------------------------------------------------------------------------------------------------

      get_instance(vInstanceName);

---------------------------------------------------------------------------------------------------------------------
-- Find out if a threshold value is passed, else we will use default.
---------------------------------------------------------------------------------------------------------------------

      IF pThreshold IS NOT NULL THEN
         vThreshold := pThreshold;
      ELSE
         OPEN threshold;
         FETCH threshold INTO vThreshold;
         CLOSE threshold;
      END IF;

      debug_message('Checking for objects which have extents > '||vThreshold);

      open extentCursor;
      LOOP
         fetch extentCursor into vSegmentName,vSegmentType,vOwner,vSegmentCount,vSegmentSize,vTablespaceName;
         exit when extentCursor%NOTFOUND;
         IF vSegmentCount >= vThreshold THEN
            vExceeded := vSegmentCount - vThreshold;
            alertlog_message(vSegmentType||' '||vOwner||'.'||vSegmentName||' has '||vSegmentCount||' extents ('||vSegmentSize||' bytes). Tablespace '||vTablespaceName);
                  send_alert('SEGMENTCOUNT',vOwner||'.'||vSegmentName,vSegmentType||' '||vOwner||'.'||vSegmentName||' has '||vSegmentCount||' extents, '||vExceeded||' more than threshold of '||vThreshold||' ('||vSegmentSize||' bytes of tablespace '||vTablespaceName||').');
         END IF;
      END LOOP;
      close extentCursor;
   END;

   Procedure resource_limits

/*

      Purpose	: Query resource usage
      Usage	: resource_limits('RESOURCE_NAME',Threshold,DebugLevel);	<- For a single resource.  
      Usage	: resource_limits;						<- For all resources.  

*/

      (
         pResourceName		IN V$RESOURCE_LIMIT.RESOURCE_NAME%TYPE DEFAULT NULL,
         pThreshold		IN NUMBER,
         pDebug			IN NUMBER DEFAULT 0
      )
      is

---------------------------------------------------------------------------------------------------------------------
--  Declarations
---------------------------------------------------------------------------------------------------------------------

      vResourceName		V$RESOURCE_LIMIT.RESOURCE_NAME%TYPE;
      vCurrentUtilization	V$RESOURCE_LIMIT.CURRENT_UTILIZATION%TYPE;
      vMaxUtilization		V$RESOURCE_LIMIT.MAX_UTILIZATION%TYPE;
      vInitialAllocation	V$RESOURCE_LIMIT.INITIAL_ALLOCATION%TYPE;
      vLimitValue		V$RESOURCE_LIMIT.LIMIT_VALUE%TYPE;
      vPctUsed			NUMBER;
      vConnection		NUMBER;
      vThreshold		NUMBER DEFAULT 85;
      

      CURSOR resourceLimitCursor IS
      SELECT
         RESOURCE_NAME,
         CURRENT_UTILIZATION,
         MAX_UTILIZATION,
         INITIAL_ALLOCATION,
         LTRIM(LIMIT_VALUE),
         ROUND(MAX_UTILIZATION*100/LIMIT_VALUE)
      FROM V$RESOURCE_LIMIT
         WHERE LIMIT_VALUE != ' UNLIMITED'
         AND MAX_UTILIZATION > 0
         AND RESOURCE_NAME LIKE vResourceName;

      CURSOR threshold IS
      SELECT 
         THRESHOLD FROM OT_MONITOR_THRESHOLDS@OT_MONITOR
      WHERE CLIENT = vInstanceName
      AND RESOURCE_TYPE = 'RESOURCE';

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set debug level which may have been passed. Otherwise, use the global.
---------------------------------------------------------------------------------------------------------------------

      IF pDebug > 0 THEN
         vDebug := pDebug;
      END IF;

---------------------------------------------------------------------------------------------------------------------
-- Get the instance name.
---------------------------------------------------------------------------------------------------------------------

      get_instance(vInstanceName);

---------------------------------------------------------------------------------------------------------------------
-- Find out if a threshold value is passed, else we will use default.
---------------------------------------------------------------------------------------------------------------------

         IF pThreshold IS NOT NULL THEN
            vThreshold := pThreshold;
         ELSE
            OPEN threshold;
            FETCH threshold INTO vThreshold;
            CLOSE threshold;
         END IF;

---------------------------------------------------------------------------------------------------------------------
-- Find out if a resource name is passed, else we will display all resources with MAX_UTILIZATION _not_ UNLIMITED.
---------------------------------------------------------------------------------------------------------------------

         IF pResourceName IS NULL THEN
            vResourceName := '%';
         ELSE
            vResourceName := UPPER(pResourceName);
         END IF;

         debug_message('=================================================================');
         if vResourceName is not null then
            debug_message('Resource allocation / usage information.');
         else
            debug_message('Resource '||vResourceName||' allocation / usage information.');
         end if;

         debug_message('=================================================================');
         debug_message('Threshold is set to '||vThreshold);

         BEGIN
            OPEN resourceLimitCursor;
   
            LOOP
               FETCH resourceLimitCursor INTO vResourceName, vCurrentUtilization, vMaxUtilization, vInitialAllocation, vLimitValue, vPctUsed;
               exit when resourceLimitCursor%NOTFOUND;
               debug_message('Resource '||vResourceName||'Current '||vCurrentUtilization||' Max '||vMaxUtilization||' Limit '||vLimitValue||' Pct used: '||vPctUsed||'%');
--               vResourceName := rpad(vResourceName,30,' ');
               IF vPctUsed >= vThreshold THEN
                  alertlog_message('Resource '||vResourceName||' is at '||vPctUsed||'% capacity.');
                  send_alert('RESOURCE',vResourceName,'Resource '||vResourceName||' is at '||vPctUsed||'% capacity ('||vCurrentUtilization||' of '||vLimitValue||'). Threshold is '||vThreshold||'%');
               END IF;
            END LOOP;

            CLOSE resourceLimitCursor;
         END;
      END;

   Procedure tablespace_capacity

/*

      Purpose   :  Query space usage info about tablespaces.
      Usage     :  tablespaces('TABLESPACE_NAME',Threshold,DebugLevel);	<- For a single tablespace.
                   tablespaces('%ESPACE_NAM%'Threshold%,DebugLevel);		<- For a pattern.
                   tablespaces;						<- For all tablespaces.

*/
      (
         pTablespaceName	IN DBA_TABLESPACES.TABLESPACE_NAME%TYPE DEFAULT NULL,
         pThreshold		IN NUMBER,
         pDebug			IN NUMBER DEFAULT 0
      )
      IS

---------------------------------------------------------------------------------------------------------------------
--  Declarations
---------------------------------------------------------------------------------------------------------------------

      vTablespaceName		DBA_TABLESPACES.TABLESPACE_NAME%TYPE;
      vThreshold		NUMBER DEFAULT 85;
      vTsName			DBA_TABLESPACES.TABLESPACE_NAME%TYPE;
      vBytesAlloc		VARCHAR2(20);
      vBytesUsed		VARCHAR2(20);
      vBytesFree		VARCHAR2(20);
      vPctUsed			NUMBER;
      vPctFree			NUMBER;
      vTmp1			VARCHAR(30);
      vTmp2			VARCHAR(30);
      vTmp3			VARCHAR(30);
      vConnection		NUMBER;

      CURSOR tablespaceCursor IS
      SELECT
         DF.TABLESPACE_NAME,
         TO_CHAR(DF.BYTES,'999,999,999,999'),
         NVL(TO_CHAR(DF.BYTES-SUM(FS.BYTES), '999,999,999,999'),
           TO_CHAR(DF.BYTES,'999,999,999,999')),
         NVL(TO_CHAR(SUM(FS.BYTES),'999,999,999,999'),'0'),
         NVL(ROUND((DF.BYTES-SUM(FS.BYTES))*100/DF.BYTES),'100'),
         NVL(ROUND(SUM(FS.BYTES)*100/DF.BYTES),'0')
      FROM DBA_FREE_SPACE FS,
         (SELECT TABLESPACE_NAME, SUM(BYTES) BYTES FROM DBA_DATA_FILES GROUP BY
         TABLESPACE_NAME ) DF
      WHERE FS.TABLESPACE_NAME LIKE vTablespaceName
      AND FS.TABLESPACE_NAME (+) = DF.TABLESPACE_NAME
      GROUP BY DF.TABLESPACE_NAME, DF.BYTES
      ORDER BY 3 DESC;

      CURSOR threshold IS
      SELECT 
         THRESHOLD FROM OT_MONITOR_THRESHOLDS@OT_MONITOR
      WHERE CLIENT = vInstanceName
      AND RESOURCE_TYPE = 'TABLESPACE';

-- begin the tablespaces procedure

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set debug level which may have been passed. Otherwise, use the global.
---------------------------------------------------------------------------------------------------------------------

      IF pDebug > 0 THEN
         vDebug := pDebug;
      END IF;

---------------------------------------------------------------------------------------------------------------------
-- Get the instance name.
---------------------------------------------------------------------------------------------------------------------

      get_instance(vInstanceName);

---------------------------------------------------------------------------------------------------------------------
-- Find out if a tablespace name is passed, else we will display all tablespaces.
---------------------------------------------------------------------------------------------------------------------

         IF pTablespaceName IS NULL THEN
            vTablespaceName := '%';
         ELSE
            vTablespaceName := UPPER(pTablespaceName);
         END IF;

---------------------------------------------------------------------------------------------------------------------
-- Find out if a threshold value is passed, else we will use default.
---------------------------------------------------------------------------------------------------------------------

         IF pThreshold IS NOT NULL THEN
            vThreshold := pThreshold;
         ELSE
            OPEN threshold;
            FETCH threshold INTO vThreshold;
            CLOSE threshold;
         END IF;

---------------------------------------------------------------------------------------------------------------------
-- Display tablespace information.
---------------------------------------------------------------------------------------------------------------------

         debug_message('=================================================================');
         if pTablespaceName is not null then
            debug_message('Tablespace space allocation / usage information.');
         else
            debug_message('Tablespace '||vTablespaceName||' space allocation / usage information.');
         end if;

         debug_message('=================================================================');
         debug_message('Threshold is set to '||vThreshold);
         debug_message('Tablespace name     Bytes alloc      Bytes used      Bytes free   %Used  %Free');
   
         BEGIN
            OPEN tablespaceCursor;
   
            LOOP
               FETCH tablespaceCursor INTO vTsName, vBytesAlloc, vBytesUsed, vBytesFree, vPctUsed, vPctFree;
               exit when tablespaceCursor%NOTFOUND;
               debug_message(vTsName||vBytesAlloc||vBytesUsed||vBytesFree||'    '||vPctUsed||'%   '||vPctFree||'%');
               IF vPctUsed >= vThreshold THEN
                  alertlog_message('Tablespace '||vTsName||' is at '||vPctUsed||'% capacity.');
                  debug_message('Tablespace '||vTsName||' is at '||vPctUsed||'% capacity.');
                  send_alert('TABLESPACE',vTsName,'Tablespace '||vTsName||' is at '||vPctUsed||'% capacity. Threshold is '||vThreshold||'%');
               END IF;
            END LOOP;

            CLOSE tablespaceCursor;
         END;
      END;

   Procedure checkall

/*

      Purpose   :  Display all info available via the ot_monitor package.
      Usage     :  checkall(DebugLevel);

*/

      (
         pDebug			IN NUMBER DEFAULT 0
      )
      IS

      CURSOR debugCursor IS
         SELECT VALUE FROM OT_MONITOR_SETTINGS@OT_MONITOR
         WHERE NAME = 'Debug';
      sessioncursor        pls_integer default dbms_sql.open_cursor;
      vDummy               pls_integer;

      BEGIN

---------------------------------------------------------------------------------------------------------------------
-- Set GLOBAL_NAMES to FALSE so we can use our OT_MONITOR database link..
---------------------------------------------------------------------------------------------------------------------

      dbms_sql.parse(sessioncursor,'alter session set global_names = FALSE',dbms_sql.native);
      vDummy := dbms_sql.execute(sessioncursor);
      dbms_sql.close_cursor(sessioncursor);

---------------------------------------------------------------------------------------------------------------------
-- Set debug level. First, check for passed value, then check server.
---------------------------------------------------------------------------------------------------------------------

         IF pDebug > 0 THEN
            vDebug := pDebug;
         ELSE
            OPEN debugCursor;
            FETCH debugCursor INTO vDebug;
            CLOSE debugCursor;
         END IF;

         debug_message('Debug is set to '||vDebug);

         get_instance(vInstanceName);
         tablespace_capacity(NULL,NULL,vDebug);
         resource_limits(NULL,NULL,vDebug);
         extent_max(NULL,vDebug);
         extent_count(NULL,vDebug);

      END;

   END OT_MONITOR_CLIENT;

-- };
